searchData={"items":[{"type":"module","title":"Tux","doc":"*Terminal User Experience*\n\nTux is a **modular**, **dependency-free** Elixir library\ndesigned for the speedy creation of elegant command line interfaces\nwhich subscribe to the philosophy *\"One module per command\"*.\n\nIts modular structure ensures its core functionalities can be overwritten\nand composed to fulfill custom needs, and having no dependencies means\nit can achieve a higher level of security by minimizing trusted parties.","ref":"Tux.html"},{"type":"module","title":"Features - Tux","doc":"Developing with the tux library involves typically the use of\nthe following featured modules:\n\n- `Tux.Dispatcher` - command registration and dispatch macros\n- `Tux.Command`    - command module behaviour\n- `Tux.Result`     - shapes for the command results\n- `Tux.Prompt`     - basic user prompting and answer parsing\n- `Tux.Error`      - user friendly error messages\n- `Tux.Config`     - reading and writing of simple configuration files\n- `Tux.Help`       - command help message construction\n- `Tux.Case`       - command testing macros","ref":"Tux.html#module-features"},{"type":"module","title":"Installation - Tux","doc":"Add `tux` to your list of dependencies in `mix.exs` and optionally update\nyour `.formatter.exs`:\n\n```\n# mix.exs\n{:tux, \"~> 0.4.0\"}\n\n# .formatter.exs\nimport_deps: [:tux]\n```","ref":"Tux.html#module-installation"},{"type":"module","title":"Examples - Tux","doc":"Here's a very short example to illustrate the mechanics of the library:\n\n1. A **command module** implements the command:\n\n```elixir\ndefmodule Demo.HelloCmd do\n  use Tux.Command\n\n  @impl true\n  def about(), do: \"Greet current user\"\n\n  @impl true\n  def main(env, args), do: {:ok, \"Hello #{env.pre.user}!\"}\nend\n```\n\n2. A **dispatcher module** associates command names with command modules,\nor groups them as sub-commands:\n\n```elixir\ndefmodule Demo do\n  use Tux.Dispatcher\n\n  # Command registration (with options)\n  cmd \"hello\", Demo.HelloCmd, preloads: [:user]\n\n  # A preload function can run prior to command execution\n  def user(env), do: System.fetch_env!(\"USER\")\nend\n```\n\n3. **Build** your elixir app as an escript executable. Note that you need to\nupdate your `mix.exs` to reflect your escript's main module.\nSee [mix escript.build](https://hexdocs.pm/mix/main/Mix.Tasks.Escript.Build.html)\nfor more details.\n\n```\n# mix.exs\ndef project do\n  [\n    escript: [main_module: Demo]\n    ...\n  ]\nend\n```\n\nGenerate the escript executable:\n\n```\n$ mix escript.build\nGenerated escript demo\n\n$ ./demo hello\nHello tuxuser!\n```\n\n4. **Extend** your app with new commands simply by creating new\ncommand modules, then register them with the dispatcher:\n\n```elixir\ndefmodule Demo.AddCmd do\n  use Tux.Command\n\n  @impl true\n  def main(env, [x, y]) do\n    {a, \"\"} = Integer.parse(x)\n    {b, \"\"} = Integer.parse(x)\n    {:ok, \"#{a} + #{b} = #{a+b}\"}\n  end\nend\n```\n\nRegister the *Demo.AddCmd* module under the *\"add\"* name:\n\n```elixir\ndefmodule Demo do\n  ...\n  cmd \"add\", Demo.AddCmd\n  cmd \"hello\", Demo.HelloCmd, preloads: [:user]\nend\n```\n\nRebuild the escript then run:\n\n```sh\n$ mix escript.build\n$ ./demo add 1 2\n1 + 2 = 3\n```","ref":"Tux.html#module-examples"},{"type":"module","title":"Overview - Tux","doc":"Tux was designed to be modular and dependency-free, all the while leveraging\nthe powerful Elixir macro system to enable the development of command line\ninterfaces with ease.","ref":"Tux.html#module-overview"},{"type":"module","title":"0. Concepts - Tux","doc":"Here's a summary of some the concepts used throughout the tux universe:\n\n  * *Command* – some functionality implemented by an executable.\n  * *Command Name* – some keyword which identifies a command.\n  * *Command Module* – a specialized module which implements a given command.\n  * *Command Dispatcher* – a specialized module responsible for grouping commands\n    and for delegating program execution to the appropriate command module\n    in response to command invocations.\n  * *Command Preloads* – optional functions dispatchers can execute prior to\n    executing commands, and whose results are made available to the command modules\n    as part of their environment.","ref":"Tux.html#module-0-concepts"},{"type":"module","title":"1. Commands - Tux","doc":"Commands expose the functionality of your program/escript to the end user.\nThey are invoked using keywords (command names), as part of your\nprogram/escript invocation.\n\nBy analogy, when you make use of the `mix` program and type `mix compile`,\nyou invoke the *compile* command of the *mix* program.\n\nTo implement commands using tux, you create *command modules* which\ncontain the business logic, then group them together under one or more\ndispatchers.\n\nHere's a the most minimal example of a command module created with tux:\n\n```\ndefmodule PingCmd do\n  use Tux.Command\n\n  @impl true\n  def main(env, args) do\n    {:ok, \"pong\"}\n  end\nend\n```\n\nCommand modules can implement other callbacks besides the `c:Tux.Command.main/2`.\nSee the `Tux.Command` behaviour for more details.\n\nAlright, so now that we have just implemented a command, how can we connect it\nto the rest of the escript? Enter dispatchers:","ref":"Tux.html#module-1-commands"},{"type":"module","title":"2. Dispatchers - Tux","doc":"```\nDispatcher Module\n   ├─── Command Module\n   ├─── Command Module\n   └─── Dispatcher Module\n            ├─── Command Module\n            └─── Command Module\n```\n\nDispatchers are modules which contain the mappings `command names -> command modules`\nand are responsible for delegating execution to the appropriate command module\nin response to a command invocation.\n\n\nWell, they actually do a little more than that, among which collecting the results\nfrom command preloads, setting up the command context, recovering\nfrom exceptions and more.\n\nA dispatcher's functionality is implemented by the `Tux.Dispatcher` module, and it can\nbe injected into your own dispatcher modules via the `__using__` macro.\nHere's an example of a dispatcher which associates the *ping* command\nwith the `PingCmd` module we've just implemented:\n\n    defmodule MyProgram do\n      use Tux.Dispatcher\n\n      # Register a new command module via a command name\n      cmd \"ping\", PingCmd\n\n      # Alternatively a command module can be registered under multiple\n      # names at once (useful for command shortcuts)\n      cmd ~w\"pong p\", PongCmd\n    end\n\n**Customizing Dispatchers** – Please read the section\n[Dispatcher Options](Tux.Dispatcher.html#module-dispatcher-options) for how to\ncustomize a dispatcher's behaviour.","ref":"Tux.html#module-2-dispatchers"},{"type":"module","title":"3. Dispatcher & Command Preloads - Tux","doc":"Oftentimes, commands need to obtain some data or run some business logic prior\nto the actual command execution, so to help with these aspects tux offers *preloads*.\n\nPreloads are functions a dispatcher can execute prior to command execution,\nand whose results can be made available to the command module in the `env` struct,\nwhich is the first argument pass to the command module's `c:Tux.Command.main/2`\ncallback.\n\nFor illustrative purposes, let's update our ping command with a preload which\ncan return the current datetime and make that available in the command output:\n\nHere's the updated dispatcher module and command registration:\n\n    defmodule MyProgram do\n      use Tux.Dispatcher\n\n      @doc \"A preload function to return the current datetime\"\n      def date(_env), do: DateTime.utc_now()\n\n      # Command registered with preload\n      cmd \"ping\", PingModule, preloads: [:date]\n    end\n\nAnd here's the command module making use of the preload's result:\n\n    defmodule PingCmd do\n      use Tux.Command\n\n      @impl true\n      def main(env, args) do\n        {:ok, \"pong #{date}\"}\n      end\n    end\n\n```\n$ mix escript.build\n$ ./program ping\npong 2024-06-07 20:36:56.687017Z\n```","ref":"Tux.html#module-3-dispatcher-command-preloads"},{"type":"module","title":"4. Results - Tux","doc":"Command modules should return the following results:\n\n  * `:ok`\n  * `:error`\n  * `{:ok, String.Chars.t()}`\n  * `{:error, String.Chars.t() | Tux.Alertable.t()}`\n\nPlease sees `String.Chars` and `Tux.Alertable` protocols for more details.","ref":"Tux.html#module-4-results"},{"type":"module","title":"5. Errors - Tux","doc":"To signal a command has failed you can return a `Tux.Result` error tuple containing a\na `Tux.Error` (or a string message) value. In such a case, this value will be converted\nand displayed nicely as a `Tux.Alert`.\n\nNow, if the business logic of a command is deeply nested, it might be easier to\nsimply raise an exception from the depths of your code, with the downside of having\nthe end user see the entire stacktrace (which might not be an issue for\nother developers), but nonetheless it can be unsightly.\n\nWith tux, you can rescue from command exceptions and display alerts instead. To this\nend, you need to construct the dispatcher module with the `rescue: true` option, and\nas a consequence, command exceptions will be shown more elegantly while still\npreserving the ability to view the entire stacktrace when the command is invoked\nwith the `--debug` flag.\n\n**Exit Codes** – A `Tux.Error` also contains an `exitcode` field, which you can\noverwrite, and it will be used as the program/escript's exit status.","ref":"Tux.html#module-5-errors"},{"type":"module","title":"6. Help - Tux","doc":"Tux includes the `Tux.Help` module to assist you in creating well-structured\ncommand help message.\n\nWhen a command implements the `help/0` callback, if the said command\nis invoked with the `-h` or `--help` flags, it will show something like this:\n\n```sh\nABOUT\n  scan - scan strings\n\nUSAGE\n  strings scan [OPTS] [ARGS]\n\nOPTIONS\n  --uppercase, u    Uppercase endpoint name\n  --lowercase, u    Lowercase endpoint name\n\nNOTES\n  Here are some additional notes to include\n  in the help message.\n```\n\nHere's the pipeline used to create the above help message:\n\n    defmodule ScanCmd do\n      use Tux.Command\n\n      @impl true\n      def about(), do: \"Manage systems\"\n\n      @impl true\n      def main(_, _), do: {:ok, \"System managed\"}\n\n      @impl true\n      def help() do\n        Help.new()\n        |> Help.about(\"scan\", \"scan strings\")\n        |> Help.usage([\"strings scan [OPTS] [ARGS]\"])\n        |> Help.options([\n          {\"--uppercase, u\", \"Uppercase target name\"},\n          {\"--lowercase, u\", \"Lowercase target name\"}\n        ])\n        |> Help.section(\"notes\", \"\"\"\n        Here are some additional notes to include\n        in the help message.\n        \"\"\")\n        |> Help.ok()\n      end\n    end","ref":"Tux.html#module-6-help"},{"type":"module","title":"7. Internals - Tux","doc":"For quick reference, here's an overview of the most important modules\nof the tux library:\n\n  * `Tux.Alert`      - module for displaying errors (stylized)\n  * `Tux.Command`    - command module behaviour\n  * `Tux.Config`     - functions for reading and writing text based config files.\n  * `Tux.Dispatcher` - macros for command creation and dispatch\n  * `Tux.Env`        - a struct to encode preload result and other info\n  * `Tux.Error`      - functions user friendly errors construction.\n  * `Tux.Exec`       - execute a command module\n  * `Tux.Help`       - functions for creating help messages.\n  * `Tux.Init`       - construct env prior to command execution\n  * `Tux.Prompt`     - functions for prompting and type conversion.\n  * `Tux.Result`     - the type returned by commands.\n  * `Tux.Show`       - functions for user-friendly output.\n\n---\n\n**More Examples** – A series of runnable examples illustrating\nthe usage of the tux library can be found in the\n[examples](https://github.com/threatfender/tux/tree/master/examples) folder\nof the source repository.","ref":"Tux.html#module-7-internals"},{"type":"behaviour","title":"Tux.Command","doc":"A command module provides the implementation for a given command,\nand it can be registered on a given dispatcher using the\n`Tux.Dispatcher.cmd/2` macro.\n\nAt a minimum, a command module needs to implement only the `c:main/2` callback.\nThe full list of callbacks is as follows:\n\n  * `c:main/2` (**required**)\n    – implements the command logic.\n  * `c:about/0` (**optional**)\n    – returns the command's short description.\n  * `c:help/0` (**optional**)\n    – returns command's help message – see `Tux.Help`.\n  * `c:show/2` (**optional**)\n    – writes final command output.\n  * `c:exit/2` (**optional**)\n    – stop the VM with an appropriate exit code.","ref":"Tux.Command.html"},{"type":"behaviour","title":"Example - Tux.Command","doc":"Here's an example of a typical command module:\n\n```\ndefmodule ExampleCmd do\n  use Tux.Command\n\n  @impl true\n  def about(), do: \"this command does something\"\n\n  @impl true\n  def main(env, args), do: {:ok, \"some result\"}\n\n  @impl true\n  def help() do\n    Help.new()\n    |> Help.about(about())\n    |> ...\n    |> Help.ok()\n  end\nend\n```","ref":"Tux.Command.html#module-example"},{"type":"macro","title":"Tux.Command.__using__/1","doc":"Implementation of the `use Tux.Command` functionality.","ref":"Tux.Command.html#__using__/1"},{"type":"callback","title":"Tux.Command.about/0","doc":"Return a very short command description which might be used\nwhen generating the help message.\n\n    defmodule ExampleCmd do\n      use Tux.Command\n\n      @impl true\n      def about(), do: \"this command does something really cool\"\n      ...\n    end","ref":"Tux.Command.html#c:about/0"},{"type":"callback","title":"Tux.Command.exit/2","doc":"Stop the Erlang VM using an appropriate exit code to reflect the command's\noutcome. This callback is implemented whenever you `use Tux.Command`,\nhowever it can be overwritten for more a custom exit logic:\n\n    defmodule ExampleCmd do\n      use Tux.Command\n\n      @impl true\n      def exit(env, result) do\n        case result do\n          :ok -> System.halt(0)\n          :error -> System.halt(1)\n        end\n\n        # System.stop(..) – slower, graceful\n        # System.halt(..) – faster, not graceful\n      end\n    end\n\nDo note that `System.stop/1` performs an asynchronous and careful stop\nof the Erlang runtime system (but takes more time), whereas `System.halt/1`\nimmediately halts the runtime system, which in turn will lead to\ncommands feeling snappier at the expense of graceful termination.","ref":"Tux.Command.html#c:exit/2"},{"type":"callback","title":"Tux.Command.help/0","doc":"Return the full help message for the command.\n\n    defmodule ExampleCmd do\n      use Tux.Command\n      alias Tux.Help\n\n      @impl true\n      def help() do\n        Help.new()\n        |> Help.usage(\"mycmd\", \"run some command\")}\n        |> Help...\n        |> Help.ok()\n      end\n    end\n\nYou can also craft the help message as a string:\n\n    defmodule ExampleCmd do\n      use Tux.Command\n\n      @impl true\n      def help() do\n        \"\"\"\n        #{bold(\"USAGE\")}\n          command [ARGS] [FLAGS]\n\n        #{bold(\"OPTIONS\")}\n          --flag1        This flag does something\n          --flag2, -f2   This flag does something else\n        \"\"\"\n        |> fn msg -> {:ok, msg} end.()\n      end\n    end","ref":"Tux.Command.html#c:help/0"},{"type":"callback","title":"Tux.Command.main/2","doc":"Implement the command logic.\n\nThis callback accepts two arguments, a `Tux.Env` and a list\nof raw arguments given to the command which you can parse with\n`OptionParser`.\n\n    defmodule ExampleCmd do\n      use Tux.Command\n\n      @impl true\n      def main(env, args) do\n        ... implement command and return result here\n      end\n    end","ref":"Tux.Command.html#c:main/2"},{"type":"callback","title":"Tux.Command.show/2","doc":"Write the final command result to the `Tux.Env`'s device.\n\nAlthough this callback is injected whenever you `use Tux.Command`,\nit is overridable and you can implement it yourself in your own command module\nfor a more custom command result printing.\n\n    defmodule ExampleCmd do\n      use Tux.Command\n\n      @impl true\n      def show(env, result) do\n        case result do\n          {:ok, data} -> IO.write(env.dev, data)\n          _ -> IO.write(env.dev, \"Something failed\")\n        end\n      end\n    end","ref":"Tux.Command.html#c:show/2"},{"type":"type","title":"Tux.Command.args/0","doc":"Command line arguments invoked with a command. These args will be available\nto the command module via the second parameter to the `c:main/2` callback.","ref":"Tux.Command.html#t:args/0"},{"type":"type","title":"Tux.Command.t/0","doc":"A command module","ref":"Tux.Command.html#t:t/0"},{"type":"behaviour","title":"Tux.Dispatcher","doc":"Dispatchers are modules which control program flow by directing the command execution\nto the appropriate command module in response to a given command/sub-command\ninvoked by the CLI's end user.","ref":"Tux.Dispatcher.html"},{"type":"behaviour","title":"Creating Dispatchers - Tux.Dispatcher","doc":"New dispatchers are created by invoking `use Tux.Dispatcher` in your own\nmodules, and then employing the various macros injected to register\n`Tux.Command` modules for their respective command names.","ref":"Tux.Dispatcher.html#module-creating-dispatchers"},{"type":"behaviour","title":"Example - Tux.Dispatcher","doc":"defmodule MyProgram do\n      use Tux.Dispatcher, opts\n\n      cmd \"ping\", PingCmd, preloads: [:auth]\n      cmd \"pong\", PongCmd, preloads: [:auth, :log]\n\n      cmd \"group\", AnotherDispatcherWithSubcommands\n\n      cmd ~w\"h hello\", HelloCmd\n      cmd ~p\".a\", CommandTriggeredByDotAPrefix\n\n      # Alternatively\n      pre [:config, :auth, :log] do\n        cmd \"ping\", PingCmd\n        cmd \"pong\", PongCmd\n      end\n    end","ref":"Tux.Dispatcher.html#module-example"},{"type":"behaviour","title":"Dispatcher Options - Tux.Dispatcher","doc":"A dispatcher's behaviour can be customized by providing a list of options:\n\n    use Tux.Dispatcher, [... rescue: true, colors: false, ...]\n\nAvailable dispatcher options are:\n\n  * `:rescue` - a boolean flag to rescue from command exceptions (default is `false`)\n  * `:device` - the `t:IO.device/0` to write the command output to\n    (default is `:stdio`)\n  * `:newlines` - a boolean flag to append new lines to outputs for nicer\n    outputs (default is `true`)\n  * `:colors` - a boolean flag to enable/disable colored outputs (default is `true`)\n  * `:preloads` - a list of functions to execute prior to executing a command module\n    (default is `[]`). The results of these preloads will be accessible\n    in the command module's `c:Tux.Command.main/2` callback as `env.pre.*`,\n    where `*` is the function name or the key (if one was specified).\n    – See [Dispatcher Preloads](#module-dispatcher-preloads) below for more details.\n  * `:exitwith` - the name of the function of the `System` module to invoke\n    in order to terminate the VM when a command returns an error result.\n    Accepted values are `:halt` and `:stop`. (default is `:halt`).\n    See [Erlang VM Exits](#module-erlang-vm-exits) below for more details.","ref":"Tux.Dispatcher.html#module-dispatcher-options"},{"type":"behaviour","title":"Erlang VM Exits - Tux.Dispatcher","doc":"When a command module returns an error, tux will terminate your program with\na non-zero exit code, so the failure can be reflected in the shell status code.\n\nBy default, in the case of errors tux terminates the program with `System.halt/1`\n(the default escript behaviour) because it is faster and we want responsive CLIs.\nHowever, if for whatever reason you need your system to gracefully come to a stop\n(say you have processes which need to perform come cleanup), you can change\nthis behaviour by specifying `exitwith: :stop` in the dispatcher's options.","ref":"Tux.Dispatcher.html#module-erlang-vm-exits"},{"type":"behaviour","title":"Dispatcher Preloads - Tux.Dispatcher","doc":"In many cases, prior to the actual execution of a command, some piece of data\nis needed (e.g. data read from a config file, etc). And since this pattern could\nrecur across many commands of a given CLI, tux introduces the concept of\n**preloads**.\n\nPreloads are functions to be executed sequentially, prior to the invocation of\na command module, and whose results will be collected under the `.pre` key of the\n`Tux.Env` struct passed to the command module's `c:Tux.Command.main/2` callback.\n\nPreloads can be declared at both the *dispatcher definition* and\n*command registration* levels, and are executed sequentially in the order in which\nthey were defined in the tux command hierarchy. Thus, the can be declared:\n\n  * At dispatcher level – using the `:preloads` dispatcher option. *(executed 1st)*\n  * At command registration level - using the `pre/2` *(2nd)* and `cmd/3` *(3rd)* macros.","ref":"Tux.Dispatcher.html#module-dispatcher-preloads"},{"type":"behaviour","title":"Preloads Format - Tux.Dispatcher","doc":"Preloads can specified using any of the following formats (Note that `key`, `fun`,\nand `mod` are atoms) and are found under the `Tux.Env`s `.pre` key:\n\n| Preload Format           | Example                       | Preload Result  |\n|:-------------------------|:------------------------------|:----------------|\n| `fun`                    | `:config`                     | `env.pre. ` |\n| `{key, fun}`             | `{c!: :config}`               | `env.pre. ` |\n| `{mod, fun, args}`       | `{MyPre, :config, []}`        | `env.pre. ` |\n| `{key, {mod, fun, args}}`| `{:mc!, {MyPre, :config, []}}`| `env.pre. ` |\n\nWhen a preload is specified using a function name, that function must exist\nin the dispatcher where that preload is being specified.","ref":"Tux.Dispatcher.html#module-preloads-format"},{"type":"behaviour","title":"Preloads Examples - Tux.Dispatcher","doc":"Here's a list of preloads specified using multiple formats at once:\n\n    defmodule MyPreloads do\n      def config(env), do: \"My config file\"\n      def config(env, :upcase), do: \"MY CONFIG FILE\"\n    end\n\n    defmodule MyDispatcher do\n      use Tux.Dispatcher\n\n      cmd \"show\", MyCmd, preloads: [:conf,\n                                    c1: :conf,\n                                    c2: {MyPreloads, :config, []},\n                                    c3: {MyPreloads, :config, [:upcase]}\n                                   ]\n\n      def conf(env), do: \"My config file\"\n    end\n\n    defmodule MyCmd do\n      use Tux.Command\n\n      def main(env, _) do\n        IO.inspect(env.pre.conf)\n        IO.inspect(env.pre.c1)\n        IO.inspect(env.pre.c2)\n        IO.inspect(env.pre.c3)\n      end\n    end\n\nAnd here's an example with preloads specified at multiple levels\nin the tux command hierarchy:\n\n    defmodule MyDispatcher do\n      use Tux.Dispatcher, preloads: [:auth, one: :inc]\n\n      def auth(_env), do: true\n      def inc(_env), do: 1\n      def inc(env, prev), do: env.pre[prev] + 1\n\n      defmodule CollectCmd do\n        use Tux.Command\n\n        @impl true\n        def main(env, _args) do\n          true = env.pre.auth\n          {:ok, \"Summing all preloads should yield three: #{env.pre.three}\"}\n        end\n      end\n\n      pre [two: {MyDispatcher, :inc, [:one]}] do\n        cmd \"collect\", CollectCmd, preloads: [three: {MyDispatcher, :inc, [:two]}]\n      end\n    end","ref":"Tux.Dispatcher.html#module-preloads-examples"},{"type":"behaviour","title":"Dispatcher Internals - Tux.Dispatcher","doc":"A dispatcher is broken up internally into various stages:\n\n  * `Tux.Init` - for command preloads execution and `Tux.Env` initialization\n  * `Tux.Exec` - for command execution\n  * `Tux.Show` - for command output display\n  * `Tux.Exit` - for command exits & stopping the VM\n  * `Tux.Help` - for dispatcher help message generation","ref":"Tux.Dispatcher.html#module-dispatcher-internals"},{"type":"function","title":"Tux.Dispatcher.__after_compile__/2","doc":"Perform sanity checks for a dispatcher's registered command names\nand associated command modules.","ref":"Tux.Dispatcher.html#__after_compile__/2"},{"type":"macro","title":"Tux.Dispatcher.__before_compile__/1","doc":"Inject the utility functions whose role is to retrieve\nthe dispatcher options at runtime (used internally).","ref":"Tux.Dispatcher.html#__before_compile__/1"},{"type":"macro","title":"Tux.Dispatcher.__using__/1","doc":"Inject the core dispatcher functionality for registration and dispatch\nof command modules.","ref":"Tux.Dispatcher.html#__using__/1"},{"type":"macro","title":"Use Options: - Tux.Dispatcher.__using__/1","doc":"* `:rescue` (boolean) - when set to true, if will rescue from command exceptions\n    and when false it will show the exception's stacktrace (default is false).\n  * `:device` (io device) - the device where to send the command results\n    (default is :stdio).\n  * `:newlines` (boolean) - whether to print the newline character with successful,\n    non-empty command results (defaults is true).","ref":"Tux.Dispatcher.html#__using__/1-use-options"},{"type":"macro","title":"Use Examples: - Tux.Dispatcher.__using__/1","doc":"defmodule GeneralDispatcher do\n      use Tux.Dispatcher\n      # ...\n    end\n\n    defmodule CatchExceptionsDispatcher do\n      use Tux.Dispatcher, rescue: true, colors: false\n      # ...\n    end","ref":"Tux.Dispatcher.html#__using__/1-use-examples"},{"type":"macro","title":"Tux.Dispatcher.cmd/3","doc":"Register a command module for a given command name in the current dispatcher.\n\nThis macro is imported automatically whenever you `use Tux.Dispacher`.","ref":"Tux.Dispatcher.html#cmd/3"},{"type":"macro","title":"Options - Tux.Dispatcher.cmd/3","doc":"* `:preloads` -  a list of functions to execute prior to command execution.\n    Preload results will be accessible in the command module's `main/2` callback\n    as `env.pre.*`. – See the [Dispatcher Preloads](#module-dispatcher-preloads)\n    section for preload specification format.","ref":"Tux.Dispatcher.html#cmd/3-options"},{"type":"macro","title":"Example - Tux.Dispatcher.cmd/3","doc":"defmodule Program do\n      use Tux.Dispatcher\n\n      cmd \"ping\", Ping\n      cmd \"cat\", Cat, preloads: [:auth, :log]\n\n      defmodule Ping do\n        def main(args), do: ...\n      end\n\n      defmodule Cat do\n        def main(args), do: ...\n      end\n    end\n\nDo note that dispatchers can be infinitely nested.\nFor example, this is valid:\n\n    defmodule Program.Design do\n      use Tux.Dispatcher\n      cmd \"intro\", Program.Design.Intro\n      cmd \"advanced\", Program.Design.Advanced\n    end\n\n    defmodule Program.Engineering do\n      use Tux.Dispatcher\n      cmd \"intro\", Program.Engineering.Intro\n      cmd \"advanced\", Program.Engineering.Advanced\n    end\n\n    defmodule Program do\n      use Tux.Dispatcher\n      cmd \"design\", Program.Design\n      cmd \"engineering\", Program.Engineering\n    end","ref":"Tux.Dispatcher.html#cmd/3-example"},{"type":"macro","title":"Command Names - Tux.Dispatcher.cmd/3","doc":"In tux you can define command names using:\n\n  * a single keyword, e.g. `ping`\n  * multiple keywords (the command will be triggered by any of the given keywords),\n    e.g. `~w(ping p)`\n  * a prefix (the command triggered by any string sequence which begins with the\n    given prefix) `~p(pi)`","ref":"Tux.Dispatcher.html#cmd/3-command-names"},{"type":"macro","title":"1. Command Names using Single Keywords - Tux.Dispatcher.cmd/3","doc":"As illustrated above a command can be defined by an exact word:\n\n    cmd \"foo\", FooCommandModule\n\nInvocation:\n\n```\n$ myprog foo\n```","ref":"Tux.Dispatcher.html#cmd/3-1-command-names-using-single-keywords"},{"type":"macro","title":"2. Command Names using Multiple Keywords - Tux.Dispatcher.cmd/3","doc":"You can also register multiple names for a command at once,\nuseful when you need multiple names (long + short) for a command.\n\n    cmd ~w(foo f), FooCommandModule\n\nNow you can invoke both the `foo` and `f` commands, which point\nto the same command module:\n\n```\n$ myprog foo\n$ myprog f\n```","ref":"Tux.Dispatcher.html#cmd/3-2-command-names-using-multiple-keywords"},{"type":"macro","title":"3. Command Names using a Prefix Keyword - Tux.Dispatcher.cmd/3","doc":"In addition to creating commands identified by keywords, tux allows you to\ndefine commands using prefixes, meaning a command module will be invoked\nin response to any given string sequence which begins with the prefix:\n\n    cmd ~p\"fo\", FooCommandModule\n\nThe command above will be triggered by any sequence of characters that\nbegin with `foo...`. For example, these are all equivalent command invocations:\n\n```\n$ myprog fo\n$ myprog foo\n$ myprog foa\n$ myprog foab\n$ myprog foabc\n```\n\nDo note that you can extract the full keyword which triggered the command\nfrom the `env.cmd` in your command module's `main` function.","ref":"Tux.Dispatcher.html#cmd/3-3-command-names-using-a-prefix-keyword"},{"type":"callback","title":"Tux.Dispatcher.help/0","doc":"Return an `{:ok, _}` tuple which contains a string or the `Tux.Help` struct, and\nwhose purpose is to encode the dispatcher's help message.\n\nThis callback is injected automatically when `use Tux.Dispatcher`\nis invoked, however it can also be overwritten.","ref":"Tux.Dispatcher.html#c:help/0"},{"type":"callback","title":"Tux.Dispatcher.main/1","doc":"Dispatch command execution to the appropriate command module registered\nfor a given command name extracted from the command line arguments.\n\nUsing its `main/1` function, a dispatcher can also serve as the main module\nof an escript.","ref":"Tux.Dispatcher.html#c:main/1"},{"type":"function","title":"Tux.Dispatcher.new/1","doc":"Return a new dispatcher struct, and override its default values\nwith those given in the `args` parameter:\n\n    iex> Tux.Dispatcher.new(rescue: true, colors: false)\n    %Tux.Dispatcher{colors: false, device: :stdio, exitwith: :halt,\n                    newlines: true, preloads: [], rescue: true}","ref":"Tux.Dispatcher.html#new/1"},{"type":"macro","title":"Tux.Dispatcher.pre/2","doc":"Register a series of commands with common preloads, thus you can replace\nthe following block:\n```\ncmd \"hello\", HelloCmd, preload: [:auth, :log]\ncmd \"bye\", ByeCmd, preload: [:auth, :log]\n```\nwith a more simplified form:\n\n```\npre [:auth, :log] do\n  cmd \"hello\", HelloCmd\n  cmd \"bye\", ByeCmd\nend\n```","ref":"Tux.Dispatcher.html#pre/2"},{"type":"function","title":"Tux.Dispatcher.sigil_p/2","doc":"A sigil which can be used to register a command module with\na given prefix instead of a complete name.\n\nThis is useful when you want to reduce typing and trigger a given command\nby a certain prefix and not a full name.","ref":"Tux.Dispatcher.html#sigil_p/2"},{"type":"function","title":"Example - Tux.Dispatcher.sigil_p/2","doc":"defmodule Program do\n      use Tux.Dispatcher\n\n      cmd ~p\".\", LetterCmd\n    end\n\nThe complete typed command will then be available in the`Tux.Env` given\nto the command module:\n\n    defmodule LetterCmd do\n      use Tux.Command\n\n      @impl true\n      def main(env, _args) do\n        case String.split(env.cmd, \".\", parts: 2) do\n          [\"\", \"a\"] -> {:ok, \"You typed A\"}\n          [\"\", \"b\"] -> {:ok, \"You typed B\"}\n          _ -> {:error, \"Invalid letter\"}\n        end\n      end\n    end\n\n```sh\n$ program .a --some-flag\nYou typed A\n\n$ program .b --some-flag\nYou typed B\n```","ref":"Tux.Dispatcher.html#sigil_p/2-example"},{"type":"type","title":"Tux.Dispatcher.cmd_module/0","doc":"Command module associated with a command name","ref":"Tux.Dispatcher.html#t:cmd_module/0"},{"type":"type","title":"Tux.Dispatcher.cmd_name/0","doc":"Command name used for command module registration","ref":"Tux.Dispatcher.html#t:cmd_name/0"},{"type":"type","title":"Tux.Dispatcher.cmd_opts/0","doc":"Command registration options.","ref":"Tux.Dispatcher.html#t:cmd_opts/0"},{"type":"type","title":"Tux.Dispatcher.device/0","doc":"An output `t:IO.device/0` such as `:stdio` or another process.","ref":"Tux.Dispatcher.html#t:device/0"},{"type":"type","title":"Tux.Dispatcher.preloads/0","doc":"Preloads are list of functions to execute prior to a command.\n\nThey can be specified in the following formats:\n\n  * as a FUNCTION name\n  * as the {KEY, FUNCTION name} tuple\n  * as the {MODULE, FUNCTION name, Args} tuple\n  * as the {KEY, {MODULE, FUNCTION name, Args}} tuple","ref":"Tux.Dispatcher.html#t:preloads/0"},{"type":"type","title":"Tux.Dispatcher.t/0","doc":"The struct which encodes the dispatcher and its options.","ref":"Tux.Dispatcher.html#t:t/0"},{"type":"module","title":"Tux.Case","doc":"Utility macros to generate test cases for dispatchers or main\nescript modules.","ref":"Tux.Case.html"},{"type":"module","title":"Examples - Tux.Case","doc":"The following example uses the `scenario/2` macro to create the\nentire `ExUnit.Case` test cases in a single step:\n\n    defmodule MyTests do\n      use Tux.Case\n\n      scenario \"check for EXACT output\",\n        using: SomeDispatcher,\n        invoke: \"ping --flag --flag\",\n        expect: [exactly: \"pong\\n\"]\n\n      scenario \"check for APPROXIMATE command output\",\n        using: SomeDispatcher,\n        invoke: \"ping --flag --flag\",\n        expect: [approx: \"pong\"]\n\n      scenario \"check command output using CUSTOM FUN\",\n        using: SomeMainModule,\n        invoke: \"ping --flag --flag\",\n        expect: fn output ->\n          assert String.contains?(output, \"pong\")\n        end\n    end\n\nThis module also includes the `execute/2` macro, which does not setup a test case\nand only asserts on the command output. This is useful if you need control\nover how you create your test cases:\n\n    defmodule MyTests do\n      use Tux.Case\n\n      test \"some test using the lower level execute macro\" do\n        execute SomeDispatcher,\n          invoke: \"ping --with --some --flags\"\n          expect: [exactly: \"pong\\n\"]\n      end\n    end","ref":"Tux.Case.html#module-examples"},{"type":"macro","title":"Tux.Case.__using__/1","doc":"This implements the functionality `use Tux.Case`.","ref":"Tux.Case.html#__using__/1"},{"type":"macro","title":"Options - Tux.Case.__using__/1","doc":"- The give options will be passed to `ExUnit.Case`.","ref":"Tux.Case.html#__using__/1-options"},{"type":"macro","title":"Tux.Case.execute/2","doc":"Execute the main function of a given dispatcher module using the given command string\nand assert on the result or run a custom function.","ref":"Tux.Case.html#execute/2"},{"type":"macro","title":"Examples - Tux.Case.execute/2","doc":"defmodule MyTests do\n      use Tux.Case\n\n      test \"check for exact pong\" do\n        execute MyProgram,\n          invoke: \"ping --with --some --flags\"\n          expect: [exactly: \"pong\\n\"]\n      end\n\n\nNote that the `expect` part given to this macro takes can take multiple\nshapes:\n\n    execute MyProgram,\n      ...\n      expect: [exactly: \"pong\\n\"]\n\n    execute MyProgram,\n      ...\n      expect: [approx: \"pong\"]\n\n    execute MyProgram,\n      ...\n      expect: fn output -> assert output =~ \"ping\" end","ref":"Tux.Case.html#execute/2-examples"},{"type":"macro","title":"Tux.Case.scenario/2","doc":"This macros wraps the `execute/2` macro with ExUnit.Case test case.","ref":"Tux.Case.html#scenario/2"},{"type":"macro","title":"Examples - Tux.Case.scenario/2","doc":"Create a new test case and check for an *exact* command output:\n\n    defmodule MyTest do\n      use Tux.Case\n\n      scenario \"exact output\",\n        using: MyProgram,\n        invoke: \"command --flag --flag\",\n        expect: [exactly: \"some output\\n\"]\n    end\n\nCreate a new test case and check for an *approximate* command output:\n\n    scenario \"partial output\",\n      ...\n      expect: [approx: \"some output\"]\n\nAlternatively, the `:expect` field can be an anonymous function which receives\none argument (the command output) and performs its own asserts:\n\n    scenario, \"custom function\",\n      ...\n      expect: fn output -> assert output =~ \"something\" end","ref":"Tux.Case.html#scenario/2-examples"},{"type":"module","title":"Tux.Colors","doc":"Basic functions wrapping a few `IO.ANSI` coloring functions\nfor writing stylized output to the terminal.\n\n`bold/2` `underline/2` + `faint/2` `gray/2`\n`red/2` `orange/2` `yellow/2` `green/2` `blue/2`\n\nWhen you `use Tux.Command` this module is imported automatically\nalong with the `Tux.Help` module.","ref":"Tux.Colors.html"},{"type":"module","title":"Usage - Tux.Colors","doc":"```\nimport Tux.Colors\n\n\"\"\"\nThis is some #{bold(\"bold text\")} illustrating the #{underline(\"usage\")}\nof Tux.Colors module. This #{red(\"word\", false)} however,\nwon't be colored because the second argument passed is false, which is useful\nwhen you want to control the colored output based on some configuration flag\npassed to the functions in this module.\n\"\"\"\n```","ref":"Tux.Colors.html#module-usage"},{"type":"function","title":"Tux.Colors.blue/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#blue/2"},{"type":"function","title":"Tux.Colors.bold/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#bold/2"},{"type":"function","title":"Tux.Colors.faint/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#faint/2"},{"type":"function","title":"Tux.Colors.gray/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#gray/2"},{"type":"function","title":"Tux.Colors.green/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#green/2"},{"type":"function","title":"Tux.Colors.orange/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#orange/2"},{"type":"function","title":"Tux.Colors.red/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#red/2"},{"type":"function","title":"Tux.Colors.underline/2","doc":"Return text with `underline` style applied","ref":"Tux.Colors.html#underline/2"},{"type":"function","title":"Tux.Colors.yellow/2","doc":"Return the `term` with style applied","ref":"Tux.Colors.html#yellow/2"},{"type":"module","title":"Tux.Config","doc":"Basic support for reading and writing `key = value` configuration files,\nwith support for ignoring comment lines (lines that start with `#`\nor `;`).","ref":"Tux.Config.html"},{"type":"module","title":"Example - Tux.Config","doc":"Say you have an existing configuration file *file.conf*:\n\n```bash\n# This is a comment line (will be ignored)\npath = /tmp\n\n# Note that all values will be parsed as strings\nrounds = 10\n\n; Windows-style comments are also ignored,\n; and so are these [sections]\n[couting]\ncount = 123\n```\n\nYou can read it with `read_file/2`:\n\n```\nwith {:ok, config} <- Tux.Config.read_file(\"file.conf\") do\n  assert config[\"path\"] == \"/tmp\"\n  assert config[\"rounds\"] == \"10\"\n  assert config[\"count\"] == \"123\"\nend\n```","ref":"Tux.Config.html#module-example"},{"type":"module","title":"Notes - Tux.Config","doc":"* All read values will be strings.\n  * All read valeus which are empty strings will be converted to nil.","ref":"Tux.Config.html#module-notes"},{"type":"function","title":"Tux.Config.read_file/2","doc":"Read a configuration file located at a given path.","ref":"Tux.Config.html#read_file/2"},{"type":"function","title":"Options: - Tux.Config.read_file/2","doc":"* `:keys` - control how configuration keys are parsed:\n    * `:strings` (default) - keys are converted to strings\n    * `:atoms` - keys are converted to atoms using `String.to_atom/1`\n    * `:atoms!` - keys are converted to atoms using `String.to_existing_atom/1`","ref":"Tux.Config.html#read_file/2-options"},{"type":"function","title":"Tux.Config.read_file!/2","doc":"Read a configuration file, but raise an exception if the operation fails.","ref":"Tux.Config.html#read_file!/2"},{"type":"function","title":"Tux.Config.write_file/2","doc":"Write configuration data to a file.\n\nThe file is created if it does not exist, and if it does exist,\nthe previous content is overwritten.","ref":"Tux.Config.html#write_file/2"},{"type":"function","title":"Tux.Config.write_file!/2","doc":"Write configuration data to a file, but raise an exception\nif the operation fails.\n\nThe file is created if it does not exist, and if it does exist,\nthe previous content is overwritten.","ref":"Tux.Config.html#write_file!/2"},{"type":"type","title":"Tux.Config.read_opts/0","doc":"Options passed to `read_file/2`.","ref":"Tux.Config.html#t:read_opts/0"},{"type":"module","title":"Tux.Env","doc":"The environment struct contains the information required by a\ndispatcher to execute the logic associated with a given CLI command,\nwhile also being the first argument to be passed to a command module's\n`Tux.Command.main` callback.","ref":"Tux.Env.html"},{"type":"module","title":"Struct Fields - Tux.Env","doc":"* `:raw` - the list of all escript args as typed by the user\n  * `:cmd` - the command name invoked by the end user which matched\n    against dispatcher's registered command names.\n    (Note that inside a dispatcher, command registration can be achieved\n    using multiple variants: a keyword, a list of keywords, a prefix. – See\n    `Tux.Dispatcher.cmd/2` for more details).\n  * `:mod` - the command module associated with the given command name.\n  * `:fun` - the command module callback to execute – .e.g. `:main` or `:help`.\n    This appropriate function is computed based on the given CLI arguments.\n    For example, when a `-h` or `--help` flag was provided, the dispatcher will\n    invoke the command module's `:help` callback, otherwise the `:main`.\n  * `:arg` - the CLI arguments (minus the command name) to pass to the command module\n    for command execution.\n  * `:dev` - the IO device where output will be written. (See `Tux.Show`).\n  * `:pre` - the map with the results from all registered preloads\n  * `:dsp` - the context/parent dispatcher module.\n  * `:new` - the boolean flag to add newlines to shown success results.\n  * `:ext` - the `System` function to invoke when stopping the VM runtime.\n    The accepted values are `:halt` or `:stop`.","ref":"Tux.Env.html#module-struct-fields"},{"type":"function","title":"Tux.Env.add/3","doc":"Set or update a struct's field, while also performing some type checks\nto validate if a given value conforms to the struct's specs.","ref":"Tux.Env.html#add/3"},{"type":"function","title":"Tux.Env.new/0","doc":"Return a new empty environment struct.","ref":"Tux.Env.html#new/0"},{"type":"type","title":"Tux.Env.arg/0","doc":"Final arguments to pass to the command module's main function","ref":"Tux.Env.html#t:arg/0"},{"type":"type","title":"Tux.Env.cmd/0","doc":"The name of the requested command captured from the CLI args","ref":"Tux.Env.html#t:cmd/0"},{"type":"type","title":"Tux.Env.dev/0","doc":"Device where to print the result","ref":"Tux.Env.html#t:dev/0"},{"type":"type","title":"Tux.Env.dsp/0","doc":"The module within which the `env.mod` was registered.","ref":"Tux.Env.html#t:dsp/0"},{"type":"type","title":"Tux.Env.ext/0","doc":"The function to use on the module `System`, when terminating\ncommand execution.","ref":"Tux.Env.html#t:ext/0"},{"type":"type","title":"Tux.Env.field/0","doc":"An environment's accepted field name.","ref":"Tux.Env.html#t:field/0"},{"type":"type","title":"Tux.Env.mfn/0","doc":"The function name of the command module to execute","ref":"Tux.Env.html#t:mfn/0"},{"type":"type","title":"Tux.Env.mod/0","doc":"The module found for the command","ref":"Tux.Env.html#t:mod/0"},{"type":"type","title":"Tux.Env.new/0","doc":"A flag which notes if a new line should be written to\nenv device when a {:ok, result} is returned","ref":"Tux.Env.html#t:new/0"},{"type":"type","title":"Tux.Env.pre/0","doc":"Preloads executed prior to command main execution","ref":"Tux.Env.html#t:pre/0"},{"type":"type","title":"Tux.Env.raw/0","doc":"Received arguments from the command line (includes the command string)","ref":"Tux.Env.html#t:raw/0"},{"type":"type","title":"Tux.Env.t/0","doc":"The environment contains all the facts necessary for command execution\nincluding any warmup artifacts.","ref":"Tux.Env.html#t:t/0"},{"type":"type","title":"Tux.Env.value/0","doc":"An environment's accepted field value.","ref":"Tux.Env.html#t:value/0"},{"type":"exception","title":"Tux.Error","doc":"Construct custom exception modules which automatically derive the\n`Tux.Alertable` protocol. The generated error type will have the\nfollowing fields:\n\n  * `:message`   - short error message which will be used as title\n  * `:details`   - detailed explanation of the error (.e.g. info about possible\n    resolutions, etc)\n  * `:exitcode` - exit code to be used at escript's termination","ref":"Tux.Error.html"},{"type":"exception","title":"Use Options - Tux.Error","doc":"When constructing new error modules with `use Tux.Error`, the following\noptions are available:\n\n  * `:message` (required) – main error message to use for the struct\n  * `:exitcode` (optional) – exit code (default is 1)","ref":"Tux.Error.html#module-use-options"},{"type":"exception","title":"Examples - Tux.Error","doc":"defmodule MyError do\n      use Tux.Error, message: \"MyError Occurred\"\n    end\n\n    defmodule MyOtherError do\n      use Tux.Error, message: \"MyError Occurred\", exitcode: 100\n    end\n\nNow you can raise or return the error from your command modules:\n\n    raise MyError\n    {:error, %MyError{...}}\n\nA useful idea is to define a `new/1` function in your error module\nwhich accepts specific information and constructs a more detailed error:\n\n    defmodule FieldError do\n      use Tux.Error, message: \"Field is Missing\"\n\n      def new(fieldname: fieldname) do\n        struct!(__MODULE__, details: \"\"\"\n        The #{red(fieldname)} is missing.\n        \"\"\")\n      end\n    end\n\nHere's how the `Tux.Alertable` protocol behaves when\ninvoked with a tux error struct:\n\n    defmodule MyError do\n      use Tux.Error, message: \"Some Error Occurred\"\n    end\n\n    error = %MyError{}\n    assert Tux.Alertable.title(error) == \"Some Error Occurred\"\n    assert Tux.Alertable.message(error) == nil\n\n    error = MyError{details: \"This error is really, really bad.\"}\n    assert Tux.Alertable.title(error) == \"Some Error Occurred\"\n    assert Tux.Alertable.message(error) == \"This error is really, really bad.\"","ref":"Tux.Error.html#module-examples"},{"type":"macro","title":"Tux.Error.__using__/1","doc":"Implement the functionality for `use Tux.Error`.","ref":"Tux.Error.html#__using__/1"},{"type":"type","title":"Tux.Error.opts/0","doc":"Options given to `use Tux.Error`","ref":"Tux.Error.html#t:opts/0"},{"type":"behaviour","title":"Tux.Help","doc":"Create beautiful looking, well-structured help messages\nfor commands or dispatchers.","ref":"Tux.Help.html"},{"type":"behaviour","title":"Features - Tux.Help","doc":"* Elegant layouts\n  * Minimal coloring\n  * Customizable","ref":"Tux.Help.html#module-features"},{"type":"behaviour","title":"Example - Tux.Help","doc":"Here's an example of a well structured command help message\nwhich you can create with tux:\n\n```sh\nABOUT\n  mng - command for controlling systems\n\nUSAGE\n  tool mng [OPTS] [ARGS]\n\nOPTIONS\n  --upcase, u        Uppercase endpoint name\n  --lowercase, l     Lowercase endpoint name\n\nNOTES\n  Here are some additional notes to include\n  in the help message.\n```","ref":"Tux.Help.html#module-example"},{"type":"behaviour","title":"Pipelined Help Messages - Tux.Help","doc":"The help message above can be created with the following `Tux.Help` pipeline:\n\n    defmodule MngCmd do\n      use Tux.Command\n\n      @impl true\n      def main(_, _), do: ...\n\n      @impl true\n      def help() do\n        Help.new()\n        |> Help.about(\"mng\", \"command for controlling systems\")\n        |> Help.usage([\"tool mng [OPTS] [ARGS]\"])\n        |> Help.options([\n          {\"--upcase, u\", \"Uppercase endpoint name\"},\n          {\"--lowercase, l\", \"Lowercase endpoint name\"},\n        ])\n        |> Help.section(\"notes\", \"\"\"\n        Here are some additional notes to include\n        in the help message.\n        \"\"\")\n        |> Help.ok()\n      end\n    end","ref":"Tux.Help.html#module-pipelined-help-messages"},{"type":"behaviour","title":"Manual Help Messages - Tux.Help","doc":"If you want complete control over your help messages, you can construct\nthem manually. Here's the code that would render the above help message:\n\n    defmodule MngCmd do\n      use Tux.Command\n\n      @impl true\n      def help() do\n        \"\"\"\n        #{bold(\"ABOUT\")}\n          mng - command for controlling systems\n        #{bold(\"\\nUSAGE\")}\n          #{green(\"tool mng [OPTS] [ARGS]\")}\n        #{bold(\"\\nOPTIONS\")}\n          #{green(\"--upcase, u\")}        Uppercase endpoint name\n          #{green(\"--lowercase, l\")}     Lowercase endpoint name\n        #{bold(\"\\nNOTES\")}\n          Here are some additional notes to include\n          in the help message.\n        end\n        \"\"\"\n        |> Help.ok()\n      end\n  end","ref":"Tux.Help.html#module-manual-help-messages"},{"type":"macro","title":"Tux.Help.__using__/1","doc":"This is used internally by the `Tux.Dispatcher` and will inject the `help/0`\ndispatcher callback for returning the help message.","ref":"Tux.Help.html#__using__/1"},{"type":"callback","title":"Tux.Help.about/0","doc":"One line, command description","ref":"Tux.Help.html#c:about/0"},{"type":"function","title":"Tux.Help.about/2","doc":"Append the about section which describes the command.\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.about(\"this is the one line cmd description\")\n    ...> |> String.Chars.to_string()\n    \"ABOUT\\n\" <>\n    \"  this is the one line cmd description\\n\"","ref":"Tux.Help.html#about/2"},{"type":"function","title":"Tux.Help.about/3","doc":"Append the about section which describes the command.","ref":"Tux.Help.html#about/3"},{"type":"function","title":"Example - Tux.Help.about/3","doc":"Specifying the command name in the about section\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.about(\"cmd\", \"this is the one line cmd description\")\n    ...> |> String.Chars.to_string()\n    \"ABOUT\\n\" <>\n    \"  cmd - this is the one line cmd description\\n\"","ref":"Tux.Help.html#about/3-example"},{"type":"function","title":"Tux.Help.add_cmd/3","doc":"","ref":"Tux.Help.html#add_cmd/3"},{"type":"function","title":"Tux.Help.align_pairs/3","doc":"A utility function to align a collection of pair items.","ref":"Tux.Help.html#align_pairs/3"},{"type":"function","title":"Example - Tux.Help.align_pairs/3","doc":"iex> Tux.Help.align_pairs([\n    ...>   {\"a\", \"some description\"},\n    ...>   {\"aaa\", \"some description\"},\n    ...>   {\"aaaaa\", \"some description\"},\n    ...> ], 0, 4)\n    [{\"a        \", \"some description\"},\n     {\"aaa      \", \"some description\"},\n     {\"aaaaa    \", \"some description\"}]\n\n    iex> Tux.Help.align_pairs([\n    ...>   {\"a\", \"some description\"},\n    ...>   {\"aaa\", \"some description\"},\n    ...>   {\"aaaaa\", \"some description\"},\n    ...> ], 2, 1)\n    [{\"  a     \", \"some description\"},\n     {\"  aaa   \", \"some description\"},\n     {\"  aaaaa \", \"some description\"}]","ref":"Tux.Help.html#align_pairs/3-example"},{"type":"function","title":"Tux.Help.build_specs/2","doc":"Build the specification for section creation","ref":"Tux.Help.html#build_specs/2"},{"type":"function","title":"Tux.Help.commands/3","doc":"Append the commands section to the help struct.","ref":"Tux.Help.html#commands/3"},{"type":"function","title":"Example - Tux.Help.commands/3","doc":"iex> alias Tux.Help\n    ...> Help.new(color: false)\n    ...> |> Help.commands([\n    ...> {\"start\", \"Start something\"},\n    ...> {\"stop\", \"Stop something\"},\n    ...> ])\n    ...> |> to_string()\n    \"COMMANDS\\n\" <>\n    \"  start    Start something\\n\" <>\n    \"  stop     Stop something\\n\"","ref":"Tux.Help.html#commands/3-example"},{"type":"function","title":"Tux.Help.has_module?/2","doc":"","ref":"Tux.Help.html#has_module?/2"},{"type":"function","title":"Tux.Help.head/2","doc":"Return the string representation of section head","ref":"Tux.Help.html#head/2"},{"type":"callback","title":"Tux.Help.help/0","doc":"Return a help struct for showing help messages","ref":"Tux.Help.html#c:help/0"},{"type":"function","title":"Tux.Help.new/1","doc":"Initialize a new help struct with the given options.","ref":"Tux.Help.html#new/1"},{"type":"function","title":"Examples - Tux.Help.new/1","doc":"iex> Tux.Help.new()\n    %Tux.Help{color: true, hue: &Tux.Colors.green/2, sections: [], upcase: true}\n\n    iex> Tux.Help.new(color: false)\n    %Tux.Help{color: false, hue: &Tux.Colors.green/2, sections: [], upcase: true}","ref":"Tux.Help.html#new/1-examples"},{"type":"function","title":"Options - Tux.Help.new/1","doc":"When constructing a new help struct you can customize it behaviour\nwith the following options:\n\n  * `:hue` (fun/2)\n    – function for coloring (*default is `Tux.Colors.green/2`*)\n  * `:color` (boolean)\n    – flag to colorize help sections (*default is true*)\n  * `:upcase` (boolean)\n    – flag to uppercase the section titles (*default is true*)","ref":"Tux.Help.html#new/1-options"},{"type":"function","title":"Tux.Help.newline/2","doc":"Add one or more newlines.","ref":"Tux.Help.html#newline/2"},{"type":"function","title":"Tux.Help.ok/1","doc":"Return the help struct wrapped in a `{:ok, ..}` tuple.\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.about(\"this is my command\")\n    ...> |> String.Chars.to_string()\n    ...> |> Tux.Help.ok()\n    {:ok, \"ABOUT\\n\" <>\n    \"  this is my command\\n\"}","ref":"Tux.Help.html#ok/1"},{"type":"function","title":"Tux.Help.options/3","doc":"Define the options section","ref":"Tux.Help.html#options/3"},{"type":"function","title":"Example - Tux.Help.options/3","doc":"iex> alias Tux.Help\n    ...> Help.new(color: false)\n    ...> |> Help.options([\n    ...> {\"--flag1\", \"flag1 description\"},\n    ...> {\"--flag2\", \"flag2 description\"},\n    ...> {\"--flag3\", \"flag3 description\"},\n    ...> ])\n    ...> |> to_string()\n    \"OPTIONS\\n\" <>\n    \"  --flag1    flag1 description\\n\" <>\n    \"  --flag2    flag2 description\\n\" <>\n    \"  --flag3    flag3 description\\n\"","ref":"Tux.Help.html#options/3-example"},{"type":"function","title":"Tux.Help.section/4","doc":"Create a custom help section with a given title and body.","ref":"Tux.Help.html#section/4"},{"type":"function","title":"Example - Tux.Help.section/4","doc":"iex> alias Tux.Help\n    ...> Help.new(color: false)\n    ...> |> Help.section(\"notes\", \"some custom section\" <> \"\\nand more\")\n    ...> |> to_string()\n    \"NOTES\\n\" <>\n    \"  some custom section\\n\" <>\n    \"  and more\"","ref":"Tux.Help.html#section/4-example"},{"type":"function","title":"Tux.Help.text/2","doc":"Append a new block of text as a new section.","ref":"Tux.Help.html#text/2"},{"type":"function","title":"Tux.Help.title/2","doc":"Append a title styled to the help sections.","ref":"Tux.Help.html#title/2"},{"type":"function","title":"Tux.Help.usage/3","doc":"Append the usage section, where a very short, general structure of the command\nis presented.","ref":"Tux.Help.html#usage/3"},{"type":"function","title":"Examples - Tux.Help.usage/3","doc":"#### – Populate usage section via plain text\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.usage([\"cmd [OPTS] [ARGS]\"])\n    ...> |> String.Chars.to_string()\n    \"USAGE\\n\" <>\n    \"  cmd [OPTS] [ARGS]\\n\"\n\n\n#### – Populate usage section using a list of commands:\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.usage([\"cmd1\", \"cmd2\", \"cmd3\"])\n    ...> |> String.Chars.to_string()\n    \"USAGE\\n\"<>\n    \"  cmd1\\n\" <>\n    \"  cmd2\\n\" <>\n    \"  cmd3\\n\"\n\n#### – Using a list of commands with one line descriptions:\n\n    iex> Tux.Help.new(color: false)\n    ...> |> Tux.Help.usage([\n    ...> {\"cmd1\", \"cmd 1 description\"},\n    ...> {\"cmd2\", \"cmd 2 description\"},\n    ...> ])\n    ...> |> String.Chars.to_string()\n    \"USAGE\\n\" <>\n    \"  cmd1    cmd 1 description\\n\" <>\n    \"  cmd2    cmd 2 description\\n\"","ref":"Tux.Help.html#usage/3-examples"},{"type":"function","title":"Options for `usage/3`: - Tux.Help.usage/3","doc":"* `:title` (string) - overwrite the default usage title with a default one\n  * `:hued` (boolean) - a flag to colorize section body (default is true)","ref":"Tux.Help.html#usage/3-options-for-usage-3"},{"type":"type","title":"Tux.Help.bin_desc/0","doc":"A program description","ref":"Tux.Help.html#t:bin_desc/0"},{"type":"type","title":"Tux.Help.bin_name/0","doc":"A program binary name","ref":"Tux.Help.html#t:bin_name/0"},{"type":"type","title":"Tux.Help.cmd_desc/0","doc":"A command description","ref":"Tux.Help.html#t:cmd_desc/0"},{"type":"type","title":"Tux.Help.cmd_name/0","doc":"A command name","ref":"Tux.Help.html#t:cmd_name/0"},{"type":"type","title":"Tux.Help.flg_desc/0","doc":"A flag description","ref":"Tux.Help.html#t:flg_desc/0"},{"type":"type","title":"Tux.Help.flg_name/0","doc":"A flag name","ref":"Tux.Help.html#t:flg_name/0"},{"type":"type","title":"Tux.Help.sec_opts/0","doc":"Options for section creation","ref":"Tux.Help.html#t:sec_opts/0"},{"type":"type","title":"Tux.Help.sec_title/0","doc":"A help message section's title","ref":"Tux.Help.html#t:sec_title/0"},{"type":"type","title":"Tux.Help.t/0","doc":"The help struct","ref":"Tux.Help.html#t:t/0"},{"type":"module","title":"Tux.Prompt","doc":"Helper functions for retrieving user input as: strings, integers, floats\nor for asking for confirmations.\n\n  * Default values will be displayed between `[...]` at the end of the prompt.\n  * Supports default values when no value (just enter) is typed.","ref":"Tux.Prompt.html"},{"type":"module","title":"Examples - Tux.Prompt","doc":"alias Tux.Prompt\n\n    with true <- Prompt.for_confirmation(\"Really continue\"),\n         {:ok, name} <- Prompt.for_string(\"What's your name\", \"Joe Doe\"),\n         {:ok, age} <- Prompt.for_string(\"What's your age\"),\n         {:ok, salary} <- Prompt.for_float(\"What's your salary\") do\n          IO.puts(\"Name: #{name}, Age: #{age}, Salary: #{salary}\")\n    end","ref":"Tux.Prompt.html#module-examples"},{"type":"function","title":"Tux.Prompt.for_confirmation/2","doc":"Prompt and parse a `yes` or `no` answer:\n\n    Tux.Prompt.for_confirmation(\"Really delete\")\n    false\n\n    Tux.Prompt.for_confirmation(\"Really delete\", :yes)\n    true","ref":"Tux.Prompt.html#for_confirmation/2"},{"type":"function","title":"Tux.Prompt.for_float/2","doc":"Prompt and parse a float answer:\n\n    Tux.Prompt.for_integer(\"What's your salary?\")\n    {:ok, 25.0}\n\n    Tux.Prompt.for_integer(\"What's your salary?\")\n    :error","ref":"Tux.Prompt.html#for_float/2"},{"type":"function","title":"Tux.Prompt.for_integer/2","doc":"Prompt and parse an integer answer:\n\n    Tux.Prompt.for_integer(\"What's your age?\")\n    {:ok, 25}\n\n    Tux.Prompt.for_integer(\"What's your age?\")\n    :error","ref":"Tux.Prompt.html#for_integer/2"},{"type":"function","title":"Tux.Prompt.for_string/2","doc":"Prompt and collect a string with the whitespace trimmed:\n\n    Tux.Prompt.for_integer(\"What's your name?\")\n    {:ok, \"Tux\"}\n\n    Tux.Prompt.for_integer(\"What's your name?\", \"Joe Doe\")\n    {:ok, \"Joe Doe\"}","ref":"Tux.Prompt.html#for_string/2"},{"type":"type","title":"Tux.Prompt.prompt/0","doc":"They type for the prompt message shown to the end user\nasking for a confirmation or some input.","ref":"Tux.Prompt.html#t:prompt/0"},{"type":"module","title":"Tux.Result","doc":"The tux result is the standard result type which must be returned by a command module,\nwhich controls what the CLI's end-user sees as output.\n\n| Result Value    | Meaning   | Command Output Behaviour                                                  |\n|-----------------|-----------|---------------------------------------------------------------------------|\n| `:ok`           | Success   | nothing will be shown                                                     |\n| `:error`        | Failure   | a `Tux.Alert` will be shown                                               |\n| `{:ok, term}`   | Success   | `term` will be shown (`term` must implement `String.Chars`).              |\n| `{:error, term}`| Failure   | a `Tux.Alert` (red)    will be shown (`term` must implement `Tux.Alertable`) |","ref":"Tux.Result.html"},{"type":"module","title":"Non-standard Results - Tux.Result","doc":"You can also return anything else you like from a command, in which case you need\nto implement your own `show/2` callback in your command module to instruct tux\nhow to print your command's results.\n\nThere are some caveats to returning non-standard command results (see below),\nhowever, case need be, here's an example of how you could do it:\n\n    defmodule Cli do\n      use Tux.Dispatcher\n\n      defmodule AddCmd do\n        @impl true\n        def about(), do: \"Add a list of numbers\"\n\n        @impl true\n        def main(_env, args) do\n          args\n          |> Enum.map(&parse/1)\n          |> Enum.reject(fn value -> value == :error end)\n          |> Enum.sum()\n        end\n\n        defp parse(string) do\n          case Integer.parse(string) do\n            {num, \"\"} -> num\n            _ -> :error\n          end\n        end\n\n        @impl true\n        def show(env, result) do\n          case \"--json\" in env.raw do\n            true ->\n                %{result: result}\n                |> to_json_string()\n                |> IO.puts()\n            false ->\n                IO.puts(result)\n          end\n        end\n      end\n\n      cmd \"add\", AddCmd\n    end\n\n    defmodule Math do\n      def main(args), do: Cli.main(args)\n    end\n\n```sh\n$ ./math add 1 2 3 4\n{\"result\": 10}\n```","ref":"Tux.Result.html#module-non-standard-results"},{"type":"module","title":"Caveats for Non-standard Results - Tux.Result","doc":"Tux determines your escript's exit code based on any given command's return value\nif it follows the `Tux.Result` format, otherwise the exit code will always be 0\n– see `Tux.Exit` – which might not be what you want if you want to signal a program error.\n\nOf course, along with your custom `show/2` implementation you can also implement\na custom `exit/2` which checks the result and stops the VM with an appropriate code.\nHere's an implementation extending the above example, which for fun\nsets the escript's exit code to the computed sum value:\n\n    defmodule AddCmd do\n      use Tux.Command\n\n      ...\n\n      @impl true\n      def exit(env, result) do\n        System.halt(result)\n        # or\n        # apply(System, env.ext, [result])\n      end\n    end","ref":"Tux.Result.html#module-caveats-for-non-standard-results"},{"type":"function","title":"Tux.Result.raise_invalid_result_type/2","doc":"A utility function for raising a RuntimeError with detailed information\nabout a non-standard command return value.","ref":"Tux.Result.html#raise_invalid_result_type/2"},{"type":"type","title":"Tux.Result.t/0","doc":"The final result returned by a command","ref":"Tux.Result.html#t:t/0"},{"type":"module","title":"Tux.Alert","doc":"An alert is a struct which encodes the textual and formatting information\nwhich can be used to display stylized error messages.","ref":"Tux.Alert.html"},{"type":"module","title":"Examples - Tux.Alert","doc":"Please be aware that the colored and embolden parts cannot be seen in this example,\nnonetheless an alert in its simplest version looks roughly something like this:\n\n```sh\n│ ERR: Command error\n```\n\nAnd with additional details:\n\n```sh\n│ ERR: Command error\n│\n│ This section has more details about the error.\n```","ref":"Tux.Alert.html#module-examples"},{"type":"function","title":"Tux.Alert.add/3","doc":"Set or update a given alert field.","ref":"Tux.Alert.html#add/3"},{"type":"function","title":"Examples - Tux.Alert.add/3","doc":"iex> alias Tux.Alert\n    ...>\n    ...> Alert.new()\n    ...> |> Alert.add(:tag, \"ERR\")\n    ...> |> Alert.add(:color, &Tux.Colors.red/1)\n    ...> |> Alert.add(:title, \"Title\")\n    ...> |> Alert.add(:message, \"Message\")\n    %Tux.Alert{color: &Tux.Colors.red/1, message: \"Message\", tag: \"ERR\", title: \"Title\"}","ref":"Tux.Alert.html#add/3-examples"},{"type":"function","title":"Tux.Alert.new/0","doc":"Return a blank alert struct","ref":"Tux.Alert.html#new/0"},{"type":"function","title":"Examples - Tux.Alert.new/0","doc":"iex> Tux.Alert.new()\n    %Tux.Alert{tag: nil, color: nil, title: nil, message: nil}","ref":"Tux.Alert.html#new/0-examples"},{"type":"type","title":"Tux.Alert.field/0","doc":"","ref":"Tux.Alert.html#t:field/0"},{"type":"type","title":"Tux.Alert.t/0","doc":"The alert struct","ref":"Tux.Alert.html#t:t/0"},{"type":"type","title":"Tux.Alert.value/0","doc":"","ref":"Tux.Alert.html#t:value/0"},{"type":"protocol","title":"Tux.Alertable","doc":"Alertable protocol is used by `Tux.Show` to display command error results\nin a user-friendly fashion using the `Tux.Alert` struct.\n\nThe protocol is implemented for the following types:\n\n  * **Strings** – alertable is implemented for strings, so that if you return\n  `{:error, \"some message\"}` from a command, the message will be shown as\n  a `Tux.Alert`.\n\n  * **Tux.Error** - alertable is also implemented for the `Tux.Error` struct\n  or any struct which is constructed with `use Tux.Error`, so that if you return\n  `{:error, %Tux.Error{}}` or `{:error, %MyDerivedError{}}` from a command,\n  the struct will be shown as a `Tux.Alert`.\n\n  * **Exceptions** - alertable is implemented for the Elixir `Exception` structs\n    and will extract the `:message` field as the alert's title. Thus, if your app\n    throws an exception and the dispatcher is using the options `rescue: true`,\n    the thrown exception will be shown as an alert.","ref":"Tux.Alertable.html"},{"type":"function","title":"Tux.Alertable.message/1","doc":"The detailed message of the alert","ref":"Tux.Alertable.html#message/1"},{"type":"function","title":"Tux.Alertable.title/1","doc":"The title of the alert","ref":"Tux.Alertable.html#title/1"},{"type":"type","title":"Tux.Alertable.t/0","doc":"All the types that implement this protocol.","ref":"Tux.Alertable.html#t:t/0"},{"type":"module","title":"Tux.Errors","doc":"Errors raised or returned by various tux components.","ref":"Tux.Errors.html"},{"type":"module","title":"Non-explainable Errors - Tux.Errors","doc":"* `ConfigReadError`\n  * `ConfigWriteError`\n  * `NotImplementedError`","ref":"Tux.Errors.html#module-non-explainable-errors"},{"type":"module","title":"Alertable Errors - Tux.Errors","doc":"* `CommandRescuedError`\n  * `CommandNotFoundError`","ref":"Tux.Errors.html#module-alertable-errors"},{"type":"behaviour","title":"Tux.Exec","doc":"Dispatcher subystem for invoking command modules, with optional rescuing\nfrom raised exceptions.\n\nThis module is injected in dispatcher at call site:\n\n    ...\n    use Tux.Exec, overrideable: true\n    ...\n\nOf course, the injected `exec/2` function can be overwritten in\nyour dispatchers:\n\n    defmodule MyDispatcher do\n      use Tux.Dispatcher\n\n      @impl true\n      def exec(env, exec_opts) do\n        ... implement custom command module execution here\n      end\n    end","ref":"Tux.Exec.html"},{"type":"macro","title":"Tux.Exec.__using__/1","doc":"Inject the `exec/2` function.","ref":"Tux.Exec.html#__using__/1"},{"type":"callback","title":"Tux.Exec.exec/2","doc":"Invoke the command module according to the information in the `env`\nand return the result, with optional recovery from exceptions.","ref":"Tux.Exec.html#c:exec/2"},{"type":"type","title":"Tux.Exec.env/0","doc":"The `Tux.Env` constructed by dispatcher for an invoked command.","ref":"Tux.Exec.html#t:env/0"},{"type":"type","title":"Tux.Exec.exec_opts/0","doc":"Execution options for command module.","ref":"Tux.Exec.html#t:exec_opts/0"},{"type":"behaviour","title":"Tux.Exit","doc":"Routines for stopping the VM with an appropriate exit code\nfrom command modules.\n\nThis logic in this module is automatically injected by dispatcher and\ncommand modules, however its callbacks can be overwritten for custom needs:\n\n    defmodule MyCmd do\n      use Tux.Command\n\n      @impl true\n      def exit(env, result), do: ...stop the VM...\n    end","ref":"Tux.Exit.html"},{"type":"function","title":"Tux.Exit.code/1","doc":"Return the correponding exit code for a given command result.\n\nThe returned exit code will be determined from the given result value,\nwhich ought to be following the `Tux.Result` format, however when non-`Tux.Result`\nvalues are given, the exit code will always be determined to be 0.","ref":"Tux.Exit.html#code/1"},{"type":"function","title":"Examples - Tux.Exit.code/1","doc":"iex> Tux.Exit.code(:ok)\n    0\n\n    iex> Tux.Exit.code({:ok, \"some term\"})\n    0\n\n    iex> Tux.Exit.code(:error)\n    1\n\n    iex> Tux.Exit.code({:error, %{exitcode: 10}})\n    10\n\n    iex> Tux.Exit.code({:error, \"some error\"})\n    1\n\n    iex> Tux.Exit.code(:anything_else_returns_0)\n    0","ref":"Tux.Exit.html#code/1-examples"},{"type":"callback","title":"Tux.Exit.exit/2","doc":"Shutdown the VM using an exit code extracted or derived\nfrom the given command result.","ref":"Tux.Exit.html#c:exit/2"},{"type":"behaviour","title":"Tux.Init","doc":"Dispatcher subsystem used internally for initialization of command environment\nand collection of preload results prior to command execution.","ref":"Tux.Init.html"},{"type":"macro","title":"Tux.Init.__using__/1","doc":"This is used internally by the `Tux.Dispatcher` to inject the init routines.","ref":"Tux.Init.html#__using__/1"},{"type":"callback","title":"Tux.Init.init/2","doc":"Initialize a new command environment from *preloads*\nand *command line arguments*.","ref":"Tux.Init.html#c:init/2"},{"type":"module","title":"Tux.Locator","doc":"Find the command module for a given command name.","ref":"Tux.Locator.html"},{"type":"function","title":"Tux.Locator.locate_cmd_module/2","doc":"Locate the corresponding command module associated with a given\ncommand name which was typed at the command line.","ref":"Tux.Locator.html#locate_cmd_module/2"},{"type":"type","title":"Tux.Locator.cmd_def/0","doc":"Command definition as registered with its dispatcher","ref":"Tux.Locator.html#t:cmd_def/0"},{"type":"type","title":"Tux.Locator.cmd_module/0","doc":"","ref":"Tux.Locator.html#t:cmd_module/0"},{"type":"type","title":"Tux.Locator.cmd_name/0","doc":"Command name given to the CLI","ref":"Tux.Locator.html#t:cmd_name/0"},{"type":"type","title":"Tux.Locator.cmd_opts/0","doc":"","ref":"Tux.Locator.html#t:cmd_opts/0"},{"type":"type","title":"Tux.Locator.cmd_regname/0","doc":"","ref":"Tux.Locator.html#t:cmd_regname/0"},{"type":"type","title":"Tux.Locator.locate_err_msg/0","doc":"Returned error messages","ref":"Tux.Locator.html#t:locate_err_msg/0"},{"type":"module","title":"Tux.Quick","doc":"Utility macros for generating hierarchical, compact skeletons of simple programs\nused for testing purposes without going the traditional route of creating\nthe modules with `Tux.Dispatcher`, `Tux.Command`, etc.\n\n  * `program/2`    - generate a top level dispatcher module\n  * `program/1`    - return the Elixir module name generated by `program/2`\n  * `dispatcher/2` - generate and register a dispatcher module\n    with a given name, inside a parent dispatcher\n  * `command/2`    - generate and register a command module\n    with a given name","ref":"Tux.Quick.html"},{"type":"module","title":"Example - Tux.Quick","doc":"Here's an example of a simple CLI tool built using this module's macros:\n\n    defmodule Tool do\n      import Tux.Quick\n\n      program \"cli\" do\n        command \"hello\", do: {:ok, \"hello world\"}\n\n        dispatcher \"ping\" do\n          command \"once\", do: {:ok, \"pong\"}\n          command \"twice\", do: {:ok, \"pong pong\"}\n        end\n      end\n\n      defdelegate main(args), to: program(\"cli\")\n    end\n\nUpdate your `mix.exs` to define the escript, build it with `mix escript.build`,\nand then finally run it:\n\n```\n$ mix escript.build\n\n$ ./tool hello\nhello\n\n$ ./tool ping once\npong\n\n$ ./tool ping twice\npong pong\n```","ref":"Tux.Quick.html#module-example"},{"type":"macro","title":"Tux.Quick.command/3","doc":"Construct a `Tux.Command` module inside a dispatcher module.\n\n    command \"ping\" do\n      {:ok, \"pong\"}\n    end\n\nYou can also supply the registration options you'd normally supply\nto the `Tux.Dispatcher.cmd/3` macro:\n\n    command \"ping\", preloads: [...] do\n      {:ok, \"pong\"}\n    end","ref":"Tux.Quick.html#command/3"},{"type":"macro","title":"Tux.Quick.dispatcher/3","doc":"Construct a `Tux.Dispatcher` module and register it with the given name as\na command module within the context of its parent dispatcher.\n\n    dispatcher \"foo\" do\n      command \"ping\", do: {:ok, \"pong\"}\n    end\n\nThis macro also accepts the usual `Tux.Dispatcher` options:\n\n    dispatcher \"foo\", rescue: true, ... do\n      ...\n    end","ref":"Tux.Quick.html#dispatcher/3"},{"type":"macro","title":"Tux.Quick.program/1","doc":"Return the name of a module which was created with `program/2`:\n\n    iex> import Tux.Quick\n    ...> program(\"mytool\")\n    :\"Elixir.mytool\"\n\n    iex> import Tux.Quick\n    ...> program(:another)\n    :\"Elixir.another\"","ref":"Tux.Quick.html#program/1"},{"type":"macro","title":"Tux.Quick.program/3","doc":"Construct a top-level dispatcher module with no registered commands,\ninside which you can define other tux structures such as dispatchers\nand commands using the `dispatcher/2` and `command/2` macros.\n\n    program \"foo\" do\n      dispatcher \"bar\" do\n        command \"ping\", do: {:ok, \"pong\"}\n      end\n    end\n\nThis macro also accepts the usual `Tux.Dispatcher` options:\n\n    program \"foo\", rescue: true do\n      ...\n    end\n\nTo return the name of module generated with this macro use `program/1`.","ref":"Tux.Quick.html#program/3"},{"type":"behaviour","title":"Tux.Show","doc":"This module is responsible for converting the `Tux.Result` returned by a command\nmodule to a string representation, and then sending it to the device (by default\nthe :stdio) defined in the `Tux.Env` struct.\n\nIn the case a command returns non-successful results (e.g. errors, warnings)\nthis module will extract the appropriate information they carry via the\n`Tux.Alertable` protocol.","ref":"Tux.Show.html"},{"type":"macro","title":"Tux.Show.__using__/1","doc":"This is used internally by the `Tux.Dispatcher` and `Tux.Command`\nto inject the show routines.","ref":"Tux.Show.html#__using__/1"},{"type":"function","title":"Tux.Show.show/2","doc":"Write a command result to the IO device.","ref":"Tux.Show.html#show/2"},{"type":"function","title":"Caveats - Tux.Show.show/2","doc":"* When the given result is `:ok`, it writes only an empty string and no newline,\n    irrespective of whether the env's newline was set to `true`,\n    because such a result means the command was successful\n    but no other information was returned.\n\n  * When the given result is `{:ok, term}`, then it will append a newline to the term\n    only if the `use Tux.Dispatcher, newline: true` was set. Note that this\n    is the default behaviour so there's no need to set `newline: true`,\n    however you can optionally prevent newlines from being written\n    by specifying `newline: false` when `use`ing the Dispatcher.\n\n  * The result `{:error, term}` will be shown as `Tux.Alert`\n\n  * Note that you can also overwrite `show/2` in your own command module\n    this way you have full control over how any particalar command\n    displays its results.","ref":"Tux.Show.html#show/2-caveats"},{"type":"callback","title":"Tux.Show.show/2","doc":"Write the result returned by a command module\nto the device defined in that command's env.","ref":"Tux.Show.html#c:show/2"},{"type":"function","title":"Tux.Show.write/3","doc":"Write a string-able item to a IO device with or without a newline.","ref":"Tux.Show.html#write/3"},{"type":"function","title":"Examples - Tux.Show.write/3","doc":"Tux.Show.write(\"something\", :stdio, true)\n    something\n    :ok\n\n    Tux.Show.write(\"something\", :stdio, false)\n    something:ok","ref":"Tux.Show.html#write/3-examples"},{"type":"type","title":"Tux.Show.device/0","doc":"A device as supported by the `IO` module.","ref":"Tux.Show.html#t:device/0"},{"type":"type","title":"Tux.Show.stringable/0","doc":"A term convertible to a string","ref":"Tux.Show.html#t:stringable/0"},{"type":"extras","title":"Tux Changelog","doc":"# Tux Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.4.0 – June 2024 - Tux Changelog","doc":"* Include dispatcher preloads in the final command preloads.\n  * Refactor and improve `Tux.Dispatcher`.\n  * Rename Tux.Error `:exit_code` to `:exitcode`.\n  * Rename `Tux.Explainable` protocol to `Tux.Alertable`.\n  * Replace demo example with recon.\n  * Rename Tux.Quick `module` macro to `program`.\n  * Remove `:warn` type from the `Tux.Result`.\n  * Add `Tux.Exit` module for controlling exits.\n  * Move all errors in the `Tux.Errors` module.\n  * Remove `Tux.Commands.NotImplemented`.\n  * Show both choices when using `Prompt.for_confirmation`.\n  * Emit `ConfigReadError` and `ConfigWriteError` on bang config functions.\n  * Add new dispatcher option `:exitwith` with possible values `:halt` or `:stop`.\n  * Exit with non-zero status when command modules return errors.","ref":"changelog.html#v0-4-0-june-2024"},{"type":"extras","title":"v0.3.5 – February 2024 - Tux Changelog","doc":"* Update `pre` macro to accept a single command.\n  * Add compile-time checks for conflicting command names.\n  * Add `cmd` support for prefixed commands via `sigil_p`.","ref":"changelog.html#v0-3-5-february-2024"},{"type":"extras","title":"v0.3.0 – January 2024 - Tux Changelog","doc":"* Update the `cmd` macro to allow registration of multiple names for a command:\n    e.g. `cmd ~w(name another_name), CmdModule`.\n  * Add support for cumulative preloads when using nested dispatchers,\n    by merging preloads down the command hierarchy.\n  * Rename the command module callback from `info/0` to `about/0`\n  * Add `Tux.Commands.NotImplemented`, which can be used as a\n    placeholder command module when one hasn't been implemented yet.\n  * Add the `pre/2` macro to simplify the registration of commands\n    with common preloads.","ref":"changelog.html#v0-3-0-january-2024"},{"type":"extras","title":"v0.2.0 – December 2023 - Tux Changelog","doc":"* Add `Tux.Init`, `Tux.Exec` & `Tux.Show` as separate stages in a command life cycle\n  * Add `Tux.Env` struct for storing preloads returns and command context\n  * Add `Tux.Colors` for writing colored output\n  * Add `Tux.Prompt` for user prompting (ints, floats, strings) and confirmations\n  * Add `Tux.Config` for reading and writing simple `key = value` config files\n  * Add `Tux.Explainable` protocol as a source for alerts\n  * Add `Tux.Alert` for displaying warnings and errors\n  * Add `Tux.Error` for creating displayable errors\n  * Add `Tux.Help` for creating help messages\n  * Add `Tux.Result` for validating returns of commands\n  * Add `Tux.Command` for creating command modules\n  * Add `Tux.Dispatcher` for creating dispatcher modules and registering\n    command modules with `cmd/3` macro.","ref":"changelog.html#v0-2-0-december-2023"},{"type":"extras","title":"v0.1.0 - Tux Changelog","doc":"* Initial commit","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}